#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

int main(int argc, char* argv[])
{
	int i, rank, np, c, local_c, local_N, N = 1000000;
	int tag = 5, source;
	double x, y;
	MPI_Status status;

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &np);

	srand48(rank);
	local_c = 0;
	local_N = N / np;

	for (i = 0; i < local_N; i++) {
		x = drand48();
		y = drand48();
		if (((x * x) + (y * y)) <= 1.0) {
			local_c++;
		}
	}

	if (rank != 0)  {
		MPI_Send(&local_c, 1, MPI_INT, 0, tag, MPI_COMM_WORLD);
	} else {
		c = local_c;

		for (source = 1; source < np; source++)  {
			MPI_Recv(&local_c, 1, MPI_INT, source, tag, MPI_COMM_WORLD, &status);
			c += local_c;
		}
		printf("Estimate of Pi = %24.16f\n", (4.0 * c) / N);
	} 

	MPI_Finalize();

	return(0);
}

